-- Dmo : nettoyage des logins

-----------------------------------------------------------------------------------------------------------------------------------------
-- Quels sont les logins qui ne servent pas ?
-----------------------------------------------------------------------------------------------------------------------------------------

-- Y-a-t'il une information quelque part ?
SELECT * FROM sys.syslogins -- accdate, non support !
SELECT * FROM sys.server_principals  -- pas d'info

/*
La date de dernier login n'est pas enregistre par SQL Server !
Quelques moyens : 
-  
- Capturer les logins avec l'audit (ou avec les vnements tendus) 
- Parser l'errorlog
- Trigger for logon : plus sr (mais en cas d'erreur plus de logon possible : dsactiver en mode DAC)
- Surveiller les utilisateurs connects avec un job priodique (sys.dm_exec_sessions) : pas fiable
*/

-----------------------------------------------------------------------------------------------------------------------------------------
-- Utiliser l'audit
-----------------------------------------------------------------------------------------------------------------------------------------
CREATE SERVER AUDIT [LoginAudit]
TO FILE 
(	FILEPATH = N'C:\Program Files\Microsoft SQL Server\MSSQL16.MSSQLSERVER\MSSQL\LOG\'
	,MAXSIZE = 100 MB
	,MAX_ROLLOVER_FILES = 10
	,RESERVE_DISK_SPACE = OFF
) WITH (QUEUE_DELAY = 1000, ON_FAILURE = CONTINUE, AUDIT_GUID = 'bf38df11-8ff5-4740-9f57-467832e0111f')
ALTER SERVER AUDIT [LoginAudit] WITH (STATE = ON)
CREATE SERVER AUDIT SPECIFICATION [LoginAuditSpec]
FOR SERVER AUDIT [LoginAudit]
--ADD (FAILED_LOGIN_GROUP),
ADD (SUCCESSFUL_LOGIN_GROUP)
WITH (STATE = ON)
GO

SELECT *
FROM sys.fn_get_audit_file((
                               SELECT log_file_path + '\LoginAudit*' FROM sys.server_file_audits
                           )
                         , DEFAULT
                         , DEFAULT
                          );

SELECT server_principal_name,MAX(event_time) AS LastLoginUTC
FROM sys.fn_get_audit_file((
                               SELECT log_file_path + '\LoginAudit*' FROM sys.server_file_audits
                           )
                         , DEFAULT
                         , DEFAULT
                          ) WHERE succeeded = 1
GROUP BY server_principal_name;


/*------------------------------------------------
[SCRIPT] Logon Trigger  
[DATABASE] master
[DESCRIPTION] Log connexions in LogonLog table
[MAJ PAR] DATAFLY - Arian Papillon
--------------------------------------------------*/
USE master;
GO
CREATE TABLE LogonLog
(
    LoginName sysname PRIMARY KEY CLUSTERED
  , Counter BIGINT
        DEFAULT (1)
  , LoginDate DATETIME2
        DEFAULT SYSDATETIME()
);
GO
CREATE TRIGGER [logoncontrolTR]
ON ALL SERVER
WITH EXECUTE AS 'sa'
FOR LOGON
AS
BEGIN
    IF NOT EXISTS
    (
        SELECT LoginName
        FROM LogonLog
        WHERE LoginName = ORIGINAL_LOGIN()
    )
        INSERT LogonLog
        (
            LoginName
        )
        VALUES
        (ORIGINAL_LOGIN());
    ELSE
        UPDATE LogonLog
        SET Counter = Counter + 1
          , LoginDate = SYSDATETIME()
        WHERE LoginName = ORIGINAL_LOGIN();
END;
GO
--Pour dsactiver : si besoin DAC ou config minimale -f
--DISABLE TRIGGER [logoncontrolTR] ON ALL SERVER
GO
SELECT LoginName
     , Counter
     , LoginDate AS LastLogin FROM LogonLog  -- Cela inclut les logins  des contained databases

-----------------------------------------------------------------------------------------------------------------------------------------
-- Mais quid des groupes ? Lesquels ne servent pas ? Utilisons xp_logininfo pour connatre l'appartenance de logins windows  des groupes...
-----------------------------------------------------------------------------------------------------------------------------------------
DROP TABLE IF EXISTS #GroupMembers
CREATE TABLE #GroupMembers
([account name] sysname, [type] CHAR(8), privilege CHAR(9), [mapped login name] sysname, [permission path] sysname)
DECLARE @sqlcmd NVARCHAR(MAX) = ''
SELECT 
@sqlcmd=@sqlcmd+'INSERT #GroupMembers EXEC xp_logininfo '''+sp.name+''',''members''; '
FROM sys.server_principals sp WHERE type_desc = 'WINDOWS_GROUP' AND sp.name NOT LIKE 'NT SERVICE\%' --(2008 compat)
BEGIN TRY
EXEC (@sqlcmd)
END TRY
BEGIN CATCH
SET @sqlcmd = ''
SELECT @sqlcmd=@sqlcmd+'INSERT #GroupMembers ([permission path],[account name],[mapped login name]) VALUES ('''+sp.name+''',''Could not obtain information'','''')'
FROM sys.server_principals sp WHERE type_desc = 'WINDOWS_GROUP' AND sp.name NOT LIKE 'NT SERVICE\%' --(2008 compat)
EXEC (@sqlcmd)
END CATCH
-- Liste users/groupes
SELECT gm.[account name],gm.[permission path],gm.[type], gm.privilege FROM sys.server_principals sp 
LEFT JOIN #GroupMembers gm ON sp.name = gm.[permission path]
WHERE sp.type_desc = 'WINDOWS_GROUP' AND sp.name NOT LIKE 'NT SERVICE\%' 

-- plusieurs cas : 
-- logins authentifis par leurs groupes uniquement
-- logins existants appartenant  un ou plusieurs groupes
SELECT LL.*,SP.name AS ExistingLogin,
(SELECT STRING_AGG(GM.[permission path] + ' (' + RTRIM(privilege) + ')',', ') FROM #GroupMembers GM WHERE LL.LoginName = GM.[account name]) AS MemberOfGroups
FROM dbo.LogonLog AS LL LEFT JOIN sys.server_principals SP ON LL.LoginName = SP.name

-- Dernire utilisation des groupes
SELECT GM.[permission path],MAX(LL.LoginDate) LastUse FROM dbo.LogonLog AS LL
LEFT JOIN #GroupMembers GM ON LL.LoginName = GM.[account name]
LEFT JOIN sys.server_principals SP ON LL.LoginName = SP.name
WHERE GM.[permission path] IS NOT NULL
GROUP BY GM.[permission path]

-----------------------------------------------------------------------------------------------------------------------------------------
-- Logins orphelins
-----------------------------------------------------------------------------------------------------------------------------------------
-- Voil mes logins existants
SELECT name, type, type_desc, SID FROM sys.server_principals 
WHERE type IN ('S','U','G') 
AND sid NOT IN (0x010100000000000512000000,0x0106000000000005500000005A048DDFF9C7430AB450D4E7477A2172AB4170F4,0x010600000000000550000000732B9753646EF90356745CB675C3AA6CD6B4D28B)
-- AUTORITE NT\Systme, NT SERVICE\Winmgmt, NT SERVICE\SQLWriter
AND name NOT LIKE '##MS|_%##' ESCAPE '|' AND name NOT LIKE 'NT Service\MSSQL%' AND name NOT LIKE 'NT Service\SQL%'

-- User2 : a t supprim AD
-- User3 : a t renomm AD en User3R

EXEC sys.sp_validatelogins  -- oui, mais...non ! User3 est renomm et peut-tre utilis !
-- code de sp_validatelogins
SELECT 'SID' = sid, 'NT Login' = loginname FROM master.dbo.syslogins
	WHERE isntname = 1 AND get_sid(loginname) IS NULL
-- recherche par loginname : et si l'utilisateur AD a t renomm ?
-- (note : get_sid n'est utilisable que par une ps systme)

-- xp_logininfo ? non plus
EXEC xp_logininfo 'DATAFLYLAB\User1','all' 
EXEC xp_logininfo 'DATAFLYLAB\User2','all' 
EXEC xp_logininfo 'DATAFLYLAB\User3','all' 

-- 1. Mthode PowerShell, la plus simple : https://www.sqlservercentral.com/blogs/finding-invalid-sql-logins
-- 2. Mthode SQL (serveur li ADSI), lourd...
-- 3. Ou CLR ?

USE [master]
GO 
EXEC master.dbo.sp_addlinkedserver @server = N'ADSI', @srvproduct=N'Active Directory Service Interfaces', @provider=N'ADSDSOObject', @datasrc=N'adsdatasource'
EXEC master.dbo.sp_addlinkedsrvlogin @rmtsrvname=N'ADSI',@useself=N'False',@locallogin=NULL,@rmtuser=N'DATAFLYLAB\Administrateur',@rmtpassword='P@ssw0rd'
GO 
-- Query
DECLARE @Domain NVARCHAR(20) = N'DATAFLYLAB';
SELECT sp.name AS login
     , sp.type_desc
     , CASE
           WHEN AD.SAMAccountName <> SUBSTRING(name, CHARINDEX('\', name) + 1, LEN(name)) THEN
               'Renamed Windows User or Group'
		   WHEN AD.SAMAccountName IS NULL THEN 'Orphaned User'
           ELSE
               ''
       END AS Warning
     , DisplayName AS AD_DisplayName
     , SAMAccountName AS AD_SAMAccountName
     , ADsPath
     , sp.sid AS Login_SID
     , ObjectSID AS AD_SID
FROM sys.server_principals sp
    LEFT JOIN OPENQUERY
              (ADSI
             , 'SELECT DisplayName,SAMAccountName,ADsPath,ObjectSID 
  FROM  ''LDAP://DATAFLYLAB.lab/DC=DATAFLYLAB,DC=lab'' 
  WHERE objectClass =  ''User'' OR objectClass = ''Group''
  '
              ) AS AD
        ON AD.ObjectSID = sp.sid
WHERE sp.type IN ( 'U', 'G' )
      AND LEFT(sp.name, LEN(@Domain) + 1) = @Domain + '\'
      AND
      (
          AD.SAMAccountName <> SUBSTRING(name, CHARINDEX('\', name) + 1, LEN(name)) 
          OR AD.ObjectSID IS NULL
      )
ORDER BY login;

-- Mais  vrifier avant de supprimer mes logins, qu'ils ne soient pas propritaires d'objets...
