-- Dmo : nettoyage des filegroups et fichiers inutiliss
-----------------------------------------------------------------------------------------------------------------------------------------
-- Allocation des filegroups 
-----------------------------------------------------------------------------------------------------------------------------------------
USE fgy
GO

SET TRAN ISOLATION LEVEL READ UNCOMMITTED;
GO

SELECT FG.name AS filegroup
	 , ISNULL(nb_tables,0) AS nb_tables
	 , ISNULL(nb_index,0) AS nb_index
     , ISNULL(pages, 0) AS allocated_object_pages
     , ISNULL(size_ko, 0) AS allocated_object_size_ko
FROM sys.filegroups AS FG
    LEFT JOIN
    (
        SELECT s.name AS FileGroupName
             , SUM(a.total_pages) AS pages
             , SUM(a.total_pages * 8 ) AS size_ko
			 , COUNT(DISTINCT t.object_id) AS nb_tables
			 , COUNT(DISTINCT CASE WHEN i.index_id > 1 THEN i.index_id ELSE NULL END) AS nb_index
        FROM sys.tables t
            JOIN sys.indexes i
                ON t.object_id = i.object_id
            JOIN sys.partitions p
                ON i.object_id = p.object_id
                   AND i.index_id = p.index_id
            JOIN sys.allocation_units a
                ON p.partition_id = a.container_id
            JOIN sys.data_spaces s
                ON a.data_space_id = s.data_space_id
            JOIN sys.filegroups fg
                ON s.data_space_id = fg.data_space_id
        WHERE t.type = 'U'
        GROUP BY s.name
    ) A
        ON FG.name = A.FileGroupName
ORDER BY FG.data_space_id

-----------------------------------------------------------------------------------------------------------------------------------------
-- Allocation des fichiers
-----------------------------------------------------------------------------------------------------------------------------------------

SELECT file_id
     , df.name
     , physical_name
     , FILEPROPERTY(df.name, 'SpaceUsed') - 8 AS allocated_pages
     , (FILEPROPERTY(df.name, 'SpaceUsed') - 8) * 8 AS allocated_size_ko
     -- -8 : header, pfs, gam, sgam, na, na, Diff_map, ml_map  (+ boot on file 1)
     , size * 8 AS file_size_ko
     , fg.name AS filegroup_name
	 , SUM((FILEPROPERTY(df.name, 'SpaceUsed') - 8)*8)  OVER (PARTITION BY df.data_space_id) AS filegroup_allocated_size
FROM sys.database_files df
    JOIN sys.filegroups fg
        ON fg.data_space_id = df.data_space_id
WHERE df.type = 0;


